/*******************************************************************************
* File Name: InputCounter.c  
* Version 2.20
*
*  Description:
*     The Counter component consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "InputCounter.h"

uint8 InputCounter_initVar = 0u;


/*******************************************************************************
* Function Name: InputCounter_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void InputCounter_Init(void) 
{
        #if (!InputCounter_UsingFixedFunction && !InputCounter_ControlRegRemoved)
            uint8 ctrl;
        #endif /* (!InputCounter_UsingFixedFunction && !InputCounter_ControlRegRemoved) */
        
        #if(!InputCounter_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 InputCounter_interruptState;
        #endif /* (!InputCounter_UsingFixedFunction) */
        
        #if (InputCounter_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            InputCounter_CONTROL &= InputCounter_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (CY_PSOC5A)
                InputCounter_CONTROL2 &= ~InputCounter_CTRL_MODE_MASK;
            #endif /* (CY_PSOC5A) */
            #if (CY_PSOC3 || CY_PSOC5LP)
                InputCounter_CONTROL3 &= ~InputCounter_CTRL_MODE_MASK;                
            #endif /* (CY_PSOC3 || CY_PSOC5LP) */
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (InputCounter_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                InputCounter_CONTROL |= InputCounter_ONESHOT;
            #endif /* (InputCounter_RunModeUsed != 0x0u) */
            
            /* Set the IRQ to use the status register interrupts */
            InputCounter_CONTROL2 |= InputCounter_CTRL2_IRQ_SEL;
            
            /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
            InputCounter_RT1 &= ~InputCounter_RT1_MASK;
            InputCounter_RT1 |= InputCounter_SYNC;     
                    
            /*Enable DSI Sync all all inputs of the Timer*/
            InputCounter_RT1 &= ~(InputCounter_SYNCDSI_MASK);
            InputCounter_RT1 |= InputCounter_SYNCDSI_EN;

        #else
            #if(!InputCounter_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = InputCounter_CONTROL & ~InputCounter_CTRL_CMPMODE_MASK;
            InputCounter_CONTROL = ctrl | InputCounter_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = InputCounter_CONTROL & ~InputCounter_CTRL_CAPMODE_MASK;
            InputCounter_CONTROL = ctrl | InputCounter_DEFAULT_CAPTURE_MODE;
            #endif /* (!InputCounter_ControlRegRemoved) */
        #endif /* (InputCounter_UsingFixedFunction) */
        
        /* Clear all data in the FIFO's */
        #if (!InputCounter_UsingFixedFunction)
            InputCounter_ClearFIFO();
        #endif /* (!InputCounter_UsingFixedFunction) */
        
        /* Set Initial values from Configuration */
        InputCounter_WritePeriod(InputCounter_INIT_PERIOD_VALUE);
        #if (!(InputCounter_UsingFixedFunction && (CY_PSOC5A)))
            InputCounter_WriteCounter(InputCounter_INIT_COUNTER_VALUE);
        #endif /* (!(InputCounter_UsingFixedFunction && (CY_PSOC5A))) */
        InputCounter_SetInterruptMode(InputCounter_INIT_INTERRUPTS_MASK);
        
        #if (!InputCounter_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            InputCounter_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            InputCounter_WriteCompare(InputCounter_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            InputCounter_interruptState = CyEnterCriticalSection();
            
            InputCounter_STATUS_AUX_CTRL |= InputCounter_STATUS_ACTL_INT_EN_MASK;
            
            /* Exit Critical Region*/
            CyExitCriticalSection(InputCounter_interruptState);
            
        #endif /* (!InputCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: InputCounter_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
*******************************************************************************/
void InputCounter_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (InputCounter_UsingFixedFunction)
        InputCounter_GLOBAL_ENABLE |= InputCounter_BLOCK_EN_MASK;
        InputCounter_GLOBAL_STBY_ENABLE |= InputCounter_BLOCK_STBY_EN_MASK;
    #endif /* (InputCounter_UsingFixedFunction) */  
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!InputCounter_ControlRegRemoved || InputCounter_UsingFixedFunction)
        InputCounter_CONTROL |= InputCounter_CTRL_ENABLE;                
    #endif /* (!InputCounter_ControlRegRemoved || InputCounter_UsingFixedFunction) */
    
}


/*******************************************************************************
* Function Name: InputCounter_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  InputCounter_initVar: Is modified when this function is called for the  
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void InputCounter_Start(void) 
{
    if(InputCounter_initVar == 0u)
    {
        InputCounter_Init();
        
        InputCounter_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    InputCounter_Enable();        
}


/*******************************************************************************
* Function Name: InputCounter_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
*******************************************************************************/
void InputCounter_Stop(void) 
{
    /* Disable Counter */
    #if(!InputCounter_ControlRegRemoved || InputCounter_UsingFixedFunction)
        InputCounter_CONTROL &= ~InputCounter_CTRL_ENABLE;        
    #endif /* (!InputCounter_ControlRegRemoved || InputCounter_UsingFixedFunction) */
    
    /* Globally disable the Fixed Function Block chosen */
    #if (InputCounter_UsingFixedFunction)
        InputCounter_GLOBAL_ENABLE &= ~InputCounter_BLOCK_EN_MASK;
        InputCounter_GLOBAL_STBY_ENABLE &= ~InputCounter_BLOCK_STBY_EN_MASK;
    #endif /* (InputCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: InputCounter_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
*******************************************************************************/
void InputCounter_SetInterruptMode(uint8 interruptsMask) 
{
    InputCounter_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: InputCounter_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
*******************************************************************************/
uint8   InputCounter_ReadStatusRegister(void) 
{
    return InputCounter_STATUS;
}


#if(!InputCounter_ControlRegRemoved)
/*******************************************************************************
* Function Name: InputCounter_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
uint8   InputCounter_ReadControlRegister(void) 
{
    return InputCounter_CONTROL;
}


/*******************************************************************************
* Function Name: InputCounter_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
void    InputCounter_WriteControlRegister(uint8 control) 
{
    InputCounter_CONTROL = control;
}

#endif  /* (!InputCounter_ControlRegRemoved) */


#if (!(InputCounter_UsingFixedFunction && (CY_PSOC5A)))
/*******************************************************************************
* Function Name: InputCounter_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
*******************************************************************************/
void InputCounter_WriteCounter(uint32 counter) \
                                   
{
    #if(InputCounter_UsingFixedFunction)
        /* assert if block is already enabled */
        CYASSERT (!(InputCounter_GLOBAL_ENABLE & InputCounter_BLOCK_EN_MASK));
        /* If block is disabled, enable it and then write the counter */
        InputCounter_GLOBAL_ENABLE |= InputCounter_BLOCK_EN_MASK;
        CY_SET_REG16(InputCounter_COUNTER_LSB_PTR, (uint16)counter);
        InputCounter_GLOBAL_ENABLE &= ~InputCounter_BLOCK_EN_MASK;
    #else
        CY_SET_REG24(InputCounter_COUNTER_LSB_PTR, counter);
    #endif /* (InputCounter_UsingFixedFunction) */
}
#endif /* (!(InputCounter_UsingFixedFunction && (CY_PSOC5A))) */


/*******************************************************************************
* Function Name: InputCounter_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) The present value of the counter.
*
*******************************************************************************/
uint32 InputCounter_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    CY_GET_REG8(InputCounter_COUNTER_LSB_PTR);
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    return (CY_GET_REG24(InputCounter_STATICCOUNT_LSB_PTR));
}


/*******************************************************************************
* Function Name: InputCounter_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint32) Present Capture value.
*
*******************************************************************************/
uint32 InputCounter_ReadCapture(void) 
{
   return ( CY_GET_REG24(InputCounter_STATICCOUNT_LSB_PTR) );  
}


/*******************************************************************************
* Function Name: InputCounter_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint32) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
*******************************************************************************/
void InputCounter_WritePeriod(uint32 period) 
{
    #if(InputCounter_UsingFixedFunction)
        CY_SET_REG16(InputCounter_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG24(InputCounter_PERIOD_LSB_PTR,period);
    #endif /* (InputCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: InputCounter_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) Present period value.
*
*******************************************************************************/
uint32 InputCounter_ReadPeriod(void) 
{
   return ( CY_GET_REG24(InputCounter_PERIOD_LSB_PTR));
}


#if (!InputCounter_UsingFixedFunction)
/*******************************************************************************
* Function Name: InputCounter_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
*******************************************************************************/
void InputCounter_WriteCompare(uint32 compare) \
                                   
{
    #if(InputCounter_UsingFixedFunction)
        CY_SET_REG16(InputCounter_COMPARE_LSB_PTR,(uint16)compare);
    #else
        CY_SET_REG24(InputCounter_COMPARE_LSB_PTR,compare);
    #endif /* (InputCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: InputCounter_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint32) Present compare value.
*
*******************************************************************************/
uint32 InputCounter_ReadCompare(void) 
{
   return ( CY_GET_REG24(InputCounter_COMPARE_LSB_PTR));
}


#if (InputCounter_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: InputCounter_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void InputCounter_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    InputCounter_CONTROL &= ~InputCounter_CTRL_CMPMODE_MASK;
    
    /* Write the new setting */
    InputCounter_CONTROL |= (compareMode << InputCounter_CTRL_CMPMODE0_SHIFT);
}
#endif  /* (InputCounter_COMPARE_MODE_SOFTWARE) */


#if (InputCounter_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: InputCounter_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void InputCounter_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    InputCounter_CONTROL &= ~InputCounter_CTRL_CAPMODE_MASK;
    
    /* Write the new setting */
    InputCounter_CONTROL |= (captureMode << InputCounter_CTRL_CAPMODE0_SHIFT);
}
#endif  /* (InputCounter_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: InputCounter_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
*******************************************************************************/
void InputCounter_ClearFIFO(void) 
{

    while(InputCounter_ReadStatusRegister() & InputCounter_STATUS_FIFONEMP)
    {
        InputCounter_ReadCapture();
    }

}
#endif  /* (!InputCounter_UsingFixedFunction) */


/* [] END OF FILE */

